#include <Wire.h>
#include <RotaryEncoder.h>
#include "config.hpp"
#include <tinyNeoPixel_Static.h>

typedef struct {
  uint8_t buttons = 0;
  uint16_t sensor1 = 0;
  uint16_t sensor2 = 0;
  uint8_t knobButton = 0;
  uint16_t knobPosition = 0;
} InputState;

RotaryEncoder *encoder = nullptr;
InputState inputState;
uint8_t pixels[NBUTTONS * 3];
tinyNeoPixel leds = tinyNeoPixel(NBUTTONS, LED_PIN, NEO_RGB, pixels);
uint8_t updateLEDs = false;

void checkPosition()
{
  encoder->tick(); // just call tick() to check the state.
}

uint8_t getButtonState() {
  uint8_t nButtonsEnabled = NBUTTONS ;
  uint8_t buttonState = 0;
  for (uint8_t iButton = 0; iButton < nButtonsEnabled; iButton++) {
    buttonState |= !digitalRead(iButton) << iButton;
  }
  return buttonState;
}

void getInputState(InputState *inputState) {
  inputState->buttons = getButtonState();
  inputState->sensor1 = analogRead(SENSOR1_PIN);
  inputState->sensor2 = analogRead(SENSOR2_PIN);
  inputState->knobPosition = encoder->getPosition();
  inputState->knobButton = !digitalRead(RENC_SWITCH);
}

void transmitDataWire() {
  Wire.write((uint8_t *)&inputState, sizeof(inputState));
}

void receiveDataWire(int numBytes) {      // the Wire library tells us how many bytes
  uint8_t command = Wire.read();

  switch (command) {
    case 'l' :
      uint8_t iLed = Wire.read();
      pixels[iLed * 3 + 0] = Wire.read();
      pixels[iLed * 3 + 1] = Wire.read();
      pixels[iLed * 3 + 2] = Wire.read();
      updateLEDs = true;
      break;
  }
}

void setup() {

  for (uint8_t iButtonPin = 0; iButtonPin < NBUTTONS; iButtonPin++)
    pinMode(iButtonPin, INPUT_PULLUP);

  pinMode(LED_PIN, OUTPUT);
  pinMode(RENC_SWITCH, INPUT_PULLUP);
  
  if (DEBUG){
    Serial.begin(115200);
  }
  
  Wire.begin(I2C_ADDRESS);
  Wire.onRequest(transmitDataWire);
  Wire.onReceive(receiveDataWire);  // give the Wire library the name of the function

  encoder = new RotaryEncoder(RENC_OUTA, RENC_OUTB, RotaryEncoder::LatchMode::TWO03);
  
  leds.clear();
  leds.show();
}

void loop() {
  encoder->tick(); // just call tick() to check the state.
  getInputState(&inputState);

  if (updateLEDs) {
    updateLEDs = false;
    leds.show(); // This sends the updated pixel color to the hardware.
  }

}
