#include <ESP8266WiFi.h>
#include <WiFiClient.h>
#include <ESP8266WebServer.h>
#include <ESP8266mDNS.h>
#include "datapixel.h"

#ifndef STASSID
#define STASSID "rgbeacon"
#define STAPSK  "flashtosee"
#endif

const char* ssid = STASSID;
const char* password = STAPSK;

ESP8266WebServer server(80);

const int led = 13;

void handleRoot() {
  digitalWrite(led, 1);
  server.send(200, "text/html",
              String("<html><head></head><body><p>hello from esp8266!</p>") +
              "<p><a href=\"./red\">Red</a></p>" +
              "<p><a href=\"./green\">Green</a></p>" +
              "<p><a href=\"./blue\">Blue</a></p></body></html>");

  digitalWrite(led, 0);
  sending = false;
}

void handleNotFound() {
  digitalWrite(led, 1);
  String message = "File Not Found\n\n";
  message += "URI: ";
  message += server.uri();
  message += "\nMethod: ";
  message += (server.method() == HTTP_GET) ? "GET" : "POST";
  message += "\nArguments: ";
  message += server.args();
  message += "\n";
  for (uint8_t i = 0; i < server.args(); i++) {
    message += " " + server.argName(i) + ": " + server.arg(i) + "\n";
  }
  server.send(404, "text/plain", message);
  digitalWrite(led, 0);
}

void setup(void) {
  pixels.begin();
  // Turn off pixels
  pixels.setPixelColor(0, 0);
  pixels.show();

  pinMode(led, OUTPUT);
  digitalWrite(led, 0);
  Serial.begin(115200);
  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);
  Serial.println("");

  // Wait for connection
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.print("Connected to ");
  Serial.println(ssid);
  Serial.print("IP address: ");
  Serial.println(WiFi.localIP());

  if (MDNS.begin("esp8266")) {
    Serial.println("MDNS responder started");
  }

  server.on("/", handleRoot);
  server.on("/red", []() {
    server.send(200, "text/plain", "Pixel is now red");
    pixels.setPixelColor(0, pixels.Color(255, 0, 0));
    pixels.show();
  });

  server.on("/green", []() {
    server.send(200, "text/plain", "Pixel is now green");
    pixels.setPixelColor(0, pixels.Color(0, 255, 0));
    pixels.show();
  });

  server.on("/blue", []() {
    server.send(200, "text/plain", "Pixel is now blue");
    pixels.setPixelColor(0, pixels.Color(0, 0, 255));
    pixels.show();
  });

  server.onNotFound(handleNotFound);

  server.begin();
  Serial.println("HTTP server started");
}

void loop(void) {
  server.handleClient();
  MDNS.update();

  // Send the IP address as separate bytes
  uint32_t ipAddress = (uint32_t)WiFi.localIP();
  uint8_t *ipAddressBytes = (uint8_t *)&ipAddress;
  uint8_t ipAddressToSend[] = {
    'S', ipAddressBytes[0], ipAddressBytes[1], ipAddressBytes[2], ipAddressBytes[3]
  };

  if (sending)
    updatePixel(ipAddressToSend, 4);
}
