/*
	Secured Esp8266 Websockets Client

	This sketch:
        1. Connects to a WiFi network
        2. Connects to a Websockets server (using WSS)
        3. Sends the websockets server a message ("Hello Server")
        4. Sends the websocket server a "ping"
        5. Prints all incoming messages while the connection is open

    NOTE:
    The sketch dosen't check or indicate about errors while connecting to
    WiFi or to the websockets server. For full example you might want
    to try the example named "Esp8266-Client" (And use the ssl methods).

	Hardware:
        For this sketch you only need an ESP8266 board.

	Created 15/02/2019
	By Gil Maimon
	https://github.com/gilmaimon/ArduinoWebsockets

*/

#include <ArduinoWebsockets.h>
#include <ESP8266WiFi.h>

const char* ssid = "WaagGuest"; //Enter SSID
const char* password = "guest@waag"; //Enter Password

const char* websockets_connection_string = "wss://socketsbay.com/wss/v2/wemos_button_test/44a9c18217525a71c63eacd8448fdf82/"; //Enter server adress

// This latest SHA1 fingerprint was updated 15.04.2021
const char echo_org_ssl_fingerprint[] PROGMEM   = "17 10 13 F1 F4 1E 8E 28 A5 65 B7 E0 B5 C1 23 F5 D3 42 1E 2A";

using namespace websockets;
#define BUTTON_PIN D2
uint8_t buttonState = HIGH;
uint8_t pButtonState = HIGH;

void onMessageCallback(WebsocketsMessage message) {
  Serial.print("Got Message: ");
  Serial.println(message.data());
}

void onEventsCallback(WebsocketsEvent event, String data) {
  if (event == WebsocketsEvent::ConnectionOpened) {
    Serial.println("Connnection Opened");
  } else if (event == WebsocketsEvent::ConnectionClosed) {
    Serial.println("Connnection Closed");
  } else if (event == WebsocketsEvent::GotPing) {
    Serial.println("Got a Ping!");
  } else if (event == WebsocketsEvent::GotPong) {
    Serial.println("Got a Pong!");
  }
}

WebsocketsClient client;
void setup() {
  pinMode(BUTTON_PIN, INPUT_PULLUP);
  Serial.begin(115200);
  // Connect to wifi
  WiFi.begin(ssid, password);

  // Wait some time to connect to wifi
  for (int i = 0; i < 10 && WiFi.status() != WL_CONNECTED; i++) {
    Serial.print(".");
    delay(1000);
  }

  // run callback when messages are received
  client.onMessage(onMessageCallback);

  // run callback when events are occuring
  client.onEvent(onEventsCallback);

  // Before connecting, set the ssl fingerprint of the server
  client.setFingerprint(echo_org_ssl_fingerprint);

  // Connect to server
  client.connect(websockets_connection_string);

  // Send a message
  client.send("Hello Server");

  // Send a ping
  client.ping();
}

void loop() {
  client.poll();
  buttonState = digitalRead(BUTTON_PIN);
  // let the websockets client check for incoming messages
  if (client.available()) {
    client.poll();
  }

  if (!buttonState && pButtonState) {
    client.send("1");
    Serial.println("ButtonPressed");
    delay(100);
  }
  if (buttonState && !pButtonState) {
    client.send("0");
    Serial.println("ButtonReleased");
    delay(100);
  }
  pButtonState = buttonState;
}
