#ifndef _MIDI_INTERFACE_H
#define _MIDI_INTERFACE_H

#include <Arduino.h>
#include "config.hpp"

class Midi {

public :
    uint8_t midiNotes[2][127];

    Midi(){
        Serial1.begin(MIDI_BAUDRATE);
        midiPanic();
    }

    void noteOn(uint8_t note, uint8_t channel, uint8_t velocity){
        midiNotes[channel-1][note] = 255;
        uint8_t midiMessage[] = {0x9, 0x90 | channel, note, velocity};
        Serial1.write(midiMessage, 4);
    }

    void noteOff(uint8_t note, uint8_t channel){
        midiNotes[channel-1][note] = 0;
        uint8_t midiMessage[] = {0x8, 0x80 | channel, note, 127};
        Serial1.write(midiMessage, 4);
    }

    void midiPanic(){
        notesOff(CHORD_CHANNEL);
        notesOff(NOTE_CHANNEL);        
    }

    void notesOff(uint8_t channel){
         for (uint8_t iNote=0; iNote<127; iNote++){ 
             if (midiNotes[channel-1][iNote] > 0)
                noteOff(iNote, channel);
        }
    }

    void sendPressure(uint8_t channel, uint8_t pressure){
        uint8_t midiMessage[] = {0xB, 0xB0 | channel, 13, pressure};
        Serial1.write(midiMessage, 4);
    }

    void chordOn(uint8_t rootNote, char quality, uint8_t velocity){
        noteOn(rootNote, CHORD_CHANNEL, velocity);
        noteOn(rootNote+7, CHORD_CHANNEL, velocity);
        noteOn(rootNote + (quality == 'm' ? 3 : 4), CHORD_CHANNEL, velocity);
    }

    void chordOff(uint8_t rootNote, char quality){
        noteOff(rootNote, CHORD_CHANNEL);
        noteOff(rootNote+7, CHORD_CHANNEL);
        noteOff(rootNote + (quality == 'm' ? 3 : 4), CHORD_CHANNEL);
    }

};

#endif