
#include <Arduino.h>

#include <Wire.h>
#include <PolledTimeout.h>
//#include <ACROBOTIC_SSD1306.h>
#include "keyboard.hpp"
#include "config.hpp"
#include "midi_interface.hpp"
#include "chordprogression.hpp"

#include "DFRobot_OLED12864.h"
#include "OLEDDisplayUi.h"

Midi *midi;
Keyboard *keyboard;
uint8_t chordIndex, lastChordIndex;
uint8_t lastRootNoteIndex , rootNoteIndex;
String noteNames[12] = {"C", "C#", "D","D#", "E","F", "F#", "G ","G#", "A ","A#", "B"};
DFRobot_OLED12864  display(0x3c);

ChordProgression *chordProgressions[] = {
  new ChordProgression("Protagonism", (char[4]){'M', 'M', 'M', 'M'}, (int8_t[4]){0, 2, 12, 14}),
  new ChordProgression("Outer Space", (char[4]){'M', 'M', 'M', 'M'}, (int8_t[4]){0, 6, 12, 18}),
  new ChordProgression("Fantastical", (char[4]){'M', 'M', 'M', 'M'}, (int8_t[4]){0, 8, 12, 20}),
  new ChordProgression("Sadness", (char[4]){'M', 'm', 'M', 'm'}, (int8_t[4]){0, 4, 12, 16}),
  new ChordProgression("Romantic", (char[4]){'M', 'm', 'M', 'm'}, (int8_t[4]){0, 5, 12, 17}),
  new ChordProgression("Wonder", (char[4]){'m', 'M', 'm', 'M'}, (int8_t[4]){0, 5, 12, 17}),
  new ChordProgression("Mystery", (char[4]){'m', 'M', 'm', 'M'}, (int8_t[4]){0, 2, 12, 14}),
  new ChordProgression("Dramatic", (char[4]){'m', 'M', 'm', 'M'}, (int8_t[4]){0, 11, 12, 23}),
  new ChordProgression("Danger", (char[4]){'m', 'm', 'm', 'm'}, (int8_t[4]){0, 6, 12, 18}),
  new ChordProgression("Evil", (char[4]){'m', 'm', 'm', 'm'}, (int8_t[4]){0, 8, 12, 20})
};

void setup() {
  Serial.begin(115200);  // start serial for output
  Wire.begin(SDA_PIN, SCL_PIN, I2C_MASTER);        // join i2c bus (address optional for master)

  // oled.init();                      // Initialze SSD1306 OLED display
  // oled.clearDisplay();              // Clear screen
  // oled.setTextXY(0, 0);    

  midi = new Midi();
  keyboard = new Keyboard(midi, chordProgressions[0]);
  
  display.init();
  display.setTextAlignment(TEXT_ALIGN_CENTER);
  display.setFont(ArialMT_Plain_16);  
  display.flipScreenVertically();

}

void loop() {
  using periodic = esp8266::polledTimeout::periodicMs;
  static periodic nextUpdate(10);
  
  if (nextUpdate) {
    keyboard->update();

    chordIndex = (keyboard->chordKeys->inputState.knobPosition % 10);
    if (chordIndex != lastChordIndex){
      keyboard->setChordProgression(chordProgressions[chordIndex]);
      display.clear();
      display.drawString(128/2, 64/2, keyboard->chordProgression->name );
      display.display();

    }
    lastChordIndex = chordIndex;

    rootNoteIndex = (keyboard->upperOctave->inputState.knobPosition % 48);
    if (rootNoteIndex != lastRootNoteIndex){
      uint8_t note = 30 + rootNoteIndex;
      keyboard->rootNote = note;
      keyboard->remapChords(keyboard->chordProgression);      
      String noteText = String(note) + ":" + noteNames[(note)%12] + (note-12)/12;
      display.clear();
      display.drawString(128/2, 64/2, noteText );
      display.display();
    }
    lastRootNoteIndex = rootNoteIndex;

  }

// int remainingTimeBudget = ui.update();

  // static periodic nextOutput(500);
  // if (nextOutput){
  //   Serial.println(keyboard->toString());
  // }

 static periodic nextOutput(20);
 if (nextOutput){
   Serial.print(keyboard->chordKeys->inputState.sensor1); Serial.print("\t");
   Serial.print( keyboard->chordKeys->inputState.sensor2); Serial.print("\t");
   Serial.print( keyboard->lowerOctave->inputState.sensor2-150); Serial.print("\t");
   Serial.print( keyboard->upperOctave->inputState.sensor2); Serial.print("\n");
 }

}